@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,18 +18,9 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../SDL_internal.h"
+#include "SDL_internal.h"
 
-#ifdef SDL_JOYSTICK_HIDAPI
 
-#ifdef SDL_LIBUSB_DYNAMIC
-#define HAVE_ENABLE_GAMECUBE_ADAPTORS
-#endif
+/* Return true if the HIDAPI should ignore a device during enumeration */
+extern bool SDL_HIDAPI_ShouldIgnoreDevice(int bus_type, Uint16 vendor_id, Uint16 product_id, Uint16 usage_page, Uint16 usage, bool libusb);
 
-#ifdef HAVE_ENABLE_GAMECUBE_ADAPTORS
-extern void SDL_EnableGameCubeAdaptors(void);
-#endif
-
-#endif /* SDL_JOYSTICK_HIDAPI */
-
-/* vi: set sts=4 ts=4 sw=4 expandtab: */
