@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,63 +18,26 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_TIMER_HAIKU
 
 #include <kernel/OS.h>
 
-#include "SDL_timer.h"
 
-static bigtime_t start;
-static SDL_bool ticks_started = SDL_FALSE;
-
-void
-SDL_TicksInit(void)
-{
-    if (ticks_started) {
-        return;
-    }
-    ticks_started = SDL_TRUE;
-
-    /* Set first ticks value */
-    start = system_time();
-}
-
-void
-SDL_TicksQuit(void)
-{
-    ticks_started = SDL_FALSE;
-}
-
-Uint64
-SDL_GetTicks64(void)
-{
-    if (!ticks_started) {
-        SDL_TicksInit();
-    }
-
-    return (Uint64) ((system_time() - start) / 1000);
-}
-
-Uint64
-SDL_GetPerformanceCounter(void)
+Uint64 SDL_GetPerformanceCounter(void)
 {
     return system_time();
 }
 
-Uint64
-SDL_GetPerformanceFrequency(void)
+Uint64 SDL_GetPerformanceFrequency(void)
 {
-    return 1000000;
+    return SDL_US_PER_SECOND;
 }
 
-void
-SDL_Delay(Uint32 ms)
+void SDL_SYS_DelayNS(Uint64 ns)
 {
-    snooze(ms * 1000);
+    snooze((bigtime_t)SDL_NS_TO_US(ns));
 }
 
-#endif /* SDL_TIMER_HAIKU */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_TIMER_HAIKU
